function val = fss(units)
%
% val = fss(units)
%
% INPUTS:
%         units : arbitrary unit label to be converted to fss
%
% OUTPUTS:
%         val   : fss value of 1 'units'
%
% This function converts a string that represents the units of some
% quantity to the proper fss value.  For example if all the units are in
% fss, the function will return 1.
%
% In addition, standard SI prefixes can be attached to any unit, including
% non-metric ones.  All labels are case-sensitive.
%
% EXAMPLES:
% The following example gives the value of value of 20 miles per hour in
% imperial units (which would be ft/s in this case).
%
%     20 * fss('mi/hr')
%
% You can convert to alternate units using division.  This converts 20
% miles per hour to meters per second.
%
%     20 * fss('mph') / fss('m/s')
%
% Note that there are no checks that your new units match your starting
% units using this method.  The list of units is quite large.  See the
% function "unit_def" for more information.  The following converts square
% furlongs to acres (the answer is 10).
%
%     fss('furlong^2 / acre')
%     fss('furlong**2/acre')
%
% The syntax is also somewhat forgiving.  For example the following will
% give the same result (number of seconds in two weeks).
%
%     fss('fortnight') / fss('s')
%     fss('fortnight / s')
%     fss('fortnight*Hz')
%     fss('fortnight Hz')
%
% The function also makes a strong attempt to recognize prefixes--even for
% non-SI units.  The following example computes the ratio of the speed of
% light to one foot per nanosecond (which is about 0.98, conveniently).
%
%     fss('c') / mks('ft/ns')
%
% But the following example also works, even though kilo-miles per
% milli-year is essentially a nonsense unit of speed.
%
%     fss('kmi / myr')
%

% Versions:
%  2012.01.05 @Derek Dalle     : Split from mks.m
%
% Copyright (c) 2010, 2012 by Derek Dalle
% BSD License

%% --- Prefixes ---

% List of prefixes.
prefix_cell = {
   'Y' , 1e24 ; ...  % yotta
   'Z' , 1e21 ; ...  % zetta
   'E' , 1e18 ; ...  % exa
   'P' , 1e15 ; ...  % peta
   'T' , 1e12 ; ...  % tera
   'G' , 1e9  ; ...  % giga
   'M' , 1e6  ; ...  % mega
   'k' , 1e3  ; ...  % kilo
   'h' , 1e2  ; ...  % hecto
   'D' , 1e1  ; ...  % deca
   'da', 1e1  ; ...  % deca
   'd' , 1e-1 ; ...  % deci
   'c' , 1e-2 ; ...  % centi
   'm' , 1e-3 ; ...  % milli
	 'μ' , 1e-6 ; ...  % micro
   '�' , 1e-6 ; ...  % micro
   'u' , 1e-6 ; ...  % micro
   'n' , 1e-9 ; ...  % nano
   'p' , 1e-12; ...  % pico
   'f' , 1e-15; ...  % femto
   'a' , 1e-18; ...  % atto
   'z' , 1e-21; ...  % zepto
   'y' , 1e-24};     % yocto
% Put columns into separate variables
prefix_name = prefix_cell(:,1);
prefix_vals = prefix_cell(:,2);


%% --- Unit definitions ---

% Basis
m    = 100 / (12 * 2.54);
kg   = (12 * 2.54) / 444.822162;
s    = 1;
K    = 9/5;
A    = 1;
rad  = 1;
mol  = 1e-3;

% Complete set of units
units_cell = unit_def(m, kg, s, K, A, rad, mol);

% Put columns into separate variables
units_name = units_cell(:,1);
units_vals = units_cell(:,2);


%% --- Calculation ---

val = unit_str(units, units_name, units_vals, ...
	prefix_name, prefix_vals);
