function u = unit_def(m, kg, s, K, A, rad, mol)
%
% u = unit_def(m, kg, s, K, A, rad, mol)
%
% INPUTS:
%         m   : meter
%         kg  : kilogram
%         s   : second
%         K   : Kelvin
%         A   : Ampere
%         rad : radian
%         mol : mol
%
% OUTPUTS:
%         u   : two-column cell array with names of units and values
%
% This function makes calculates the value of a full set of units given
% several basis units.  The full list of units that are declared is quite
% extensive.  See the source code of this file for the full list of
% recognized symbols and what each represents.
%

% Versions:
%  2011.01.05 @Derek Dalle     : Extracted from mks.m and rearranged
%
% Copyright (c) 2010, 2012 by Derek Dalle
% BSD License


%% --- Unit preparation ---
% This section computes convenient unit values before making the table.
% Number
kmol = 1e3*mol;
NA   = 6.022141793e23/mol;
% Angular measure
deg  = pi/180*rad;
rev  = 2*pi * rad;
% Length units
in   = 0.0254*m;
ft   = 12*in;
yrd  = 3*ft;
mi   = 5280*ft;
nmi  = 1852*m;
% Area units
acre = 1/640*mi^2;
ha   = 10000*m^2;
% Volume
L    = 0.001*m^3;
gal  = 231*in^3;
% Time
hr   = 3600*s;
day  = 24*hr;
yr   = 365.242199*day;
a    = 365.25*day;
% Speed
kn   = nmi/hr;
% Astronomical units
c    = 299792458*m/s;
ly   = c*a;
AU   = 1.4960e11;
pc   = cot(pi/(3600*180))*AU/ly;
% Acceleration
g0   = 9.80665*m/s^2;
% Force
N    = kg*m/s^2;
lb   = 4.44822162*N;
kgf  = g0*kg;
% Mass
slug = lb*s^2/ft;
lbm  = 0.45359237*kg;
% Pressure
Pa   = N/m^2;
atm  = 101325*Pa;
torr = atm/760;
psi  = lb/in^2;
% Energy
J    = N*m;
eV   = 1.602176531*J;
cal  = 4.184*J;
Btu  = 1054.35026444*J;
% Power
W    = J/s;
hp   = 550*ft*lb/s;
% Temperature (no offsets)
R    = 5/9*K;
% Charge
C    = A*s;
e    = 1.6021764874e-19*C;
% Electric potential
V    = J/C;
% Capacitance
F    = C/V;
% Resistance and conductance
ohm  = V/A;
mho  = A/V;


%% --- List of units ---
% 
% List of basic units and their values in mks.
u = {...
   '1'        , 1        ; ...  % number
   'a'        , a        ; ...  % annum
   'A'        , A        ; ...  % Ampere
   'acre'     , acre     ; ...  % acre
   'atm'      , atm      ; ...  % atmosphere
   'AU'       , AU       ; ...  % astronomical unit
   'bar'      , 1e5*Pa   ; ...  % bar
   'btu'      , Btu      ; ...  % British thermal unit
   'Btu'      , Btu      ; ...  % British thermal unit
   'BTU'      , Btu      ; ...  % British thermal unit
   'C'        , C        ; ...  % Coulomb
   'c'        , c        ; ...  % speed of light
   'cal'      , cal      ; ...  % calorie
   'day'      , day      ; ...  % day
   'days'     , day      ; ...  % days
   'deg'      , deg      ; ...  % degree
   'dyn'      , 1e-5*N   ; ...  % dyne
   'e'        , e        ; ...  % charge of an electron
   'erg'      , 1e-7*J   ; ...  % erg
   'eV'       , eV       ; ...  % electron Volt
   'F'        , F        ; ...  % Faraday
   'fortnight', 14*day   ; ...  % fortnight
   'ft'       , ft       ; ...  % foot
   'furlong'  , 660*ft   ; ...  % furlong
   'gal'      , gal      ; ...  % gallon
   'g'        , 1e-3*kg  ; ...  % gram
   'g0'       , g0       ; ...  % reference acceleration due to gravity
   'ha'       , ha       ; ...  % hectare
   'hp'       , hp       ; ...  % horsepower
   'hr'       , hr       ; ...  % hour
   'Hz'       , 1/s      ; ...  % Herz
   'in'       , in       ; ...  % inch
   'J'        , J        ; ...  % Joule
   'K'        , K        ; ...  % Kelvin
   'kgf'      , kgf      ; ...  % kilogram force
   'kmol'     , kmol     ; ...  % kmol
   'kn'       , kn       ; ...  % knot (nautical mile per hour)
   'ksi'      , 1e3*psi  ; ...  % kilopounds per square inch
   'kt'       , kn       ; ...  % knot (nautical mile per hour)
   'kts'      , kn       ; ...  % knot (nautical mile per hour)
   'L'        , L        ; ...  % Litre
   'lb'       , lb       ; ...  % pound (force)
   'lbf'      , lb       ; ...  % pound force
   'lbm'      , lbm      ; ...  % pound mass
   'ly'       , ly       ; ...  % light year
   'm'        , m        ; ...  % meter
   'mho'      , mho      ; ...  % Siemens
   'mi'       , mi       ; ...  % mile
   'min'      , 60*s     ; ...  % minute
   'mol'      , mol      ; ...  % mol
   'mph'      , mi/hr    ; ...  % mile per hour
   'N'        , N        ; ...  % Newton
   'NA'       , NA       ; ...  % Avagardro's number
   'nmi'      , nmi      ; ...  % nautical mile
   'ohm'      , ohm      ; ...  % Ohm
   'Ohm'      , ohm      ; ...  % Ohm
   'oz'       , lb/16    ; ...  % ounce (weight)
   'Pa'       , Pa       ; ...  % Pascal
   'pc'       , pc       ; ...  % parsec
   'psi'      , psi      ; ...  % pounds per square inch
   'R'        , R        ; ...  % Rankine
   'rad'      , rad      ; ...  % radian
	 'rev'      , rev      ; ...  % revolution
	 'rpm'      , rev/60/s ; ...  % revolutions per minute
   's'        , s        ; ...  % second
   'S'        , mho      ; ...  % Siemens
   'slug'     , slug     ; ...  % slug
   'st'       , 14*lbm   ; ...  % stone
   'torr'     , torr     ; ...  % torr
   'Torr'     , torr     ; ...  % torr
   'V'        , V        ; ...  % Volt
   'W'        , W        ; ...  % Watt
   'Wh'       , W*hr     ; ...  % Watt-hour
   'yd'       , yrd      ; ...  % yard
   'year'     , yr       ; ...  % year
   'years'    , yr       ; ...  % year
   'yr'       , yr       ; ...  % year
   'yrd'      , yrd      ; ...  % yard
   'yrs'      , yr       };     % year
 